@@ -1,10 +1,10 @@
 ##----------------------------------------------------------------------------
 ## tel.pm
-## Version 0.1
-## Copyright(c) 2016-2018 Jacques Deguest
+## Version 0.5.1
+## Copyright(c) 2016-2019 Jacques Deguest
 ## Author: Jacques Deguest <jack@deguest.jp>
 ## Created 2016/02/12
-## Modified 2018/03/31
+## Modified 2019/08/26
 ## All rights reserved.
 ## 
 ## This program is free software; you can redistribute it and/or modify it 
@@ -14,16 +14,12 @@ package URI::tel;
 BEGIN
 {
 	use strict;
-    our( $VERSION, @ISA, @EXPORT, @EXPORT_OK, %EXPORT_TAGS );
-    our( $VERBOSE, $DEBUG, $ERROR );
+	use parent 'URI';
+    our( $VERSION, $VERBOSE, $DEBUG, $ERROR );
     our( $RESERVED, $MARK, $UNRESERVED, $PCT_ENCODED, $URIC, $ALPHA, $DIGIT, $ALPHANUM, $HEXDIG );
     our( $PARAM_UNRESERVED, $VISUAL_SEPARATOR, $PHONEDIGIT, $GLOBAL_NUMBER_DIGITS, $PARAMCHAR, $DOMAINLABEL, $TOPLABEL, $DOMAINNAME, $DESCRIPTOR, $PNAME, $PVALUE, $PARAMETER, $EXTENSION, $ISDN_SUBADDRESS, $CONTEXT, $PAR, $PHONEDIGIT_HEX, $GLOBAL_NUMBER, $LOCAL_NUMBER, $OTHER, $TEL_SUBSCRIBER, $TEL_URI );
-    our( $COUNTRIES );
-    @ISA         = qw( );
-    @EXPORT      = qw( );
-    %EXPORT_TAGS = ();
-    @EXPORT_OK   = qw( );
-    $VERSION     = '0.2.2';
+    our( $COUNTRIES, $IDD_RE );
+    $VERSION     = '0.5.1';
 	use overload ('""'     => 'as_string',
 				  '=='     => sub { _obj_eq(@_) },
 				  '!='     => sub { !_obj_eq(@_) },
@@ -125,6 +121,7 @@ sub new
 	{
 		$temp->{ 'type' } = 'local';
 		@names = qw( all subscriber params1 last_param1 ignore5 ignore6 context params2 last_param2 ignore10 ignore11 );
+		$temp->{ '_has_context_param' } = 1 if( length( $matches[6] ) );
 	}
 	elsif( @matches = $str =~ /^((?:tel\:)?$OTHER)$/ )
 	{
@@ -141,10 +138,37 @@ sub new
 		$ERROR = "Unknown telephone number '$str'.";
 		warn( $ERROR );
 	}
+	
+	if( $str =~ /^(?:tel\:)?\+/ )
+	{
+		## Extract the global idd
+		$this->_load_countries;
+		( my $str2 = $str ) =~ s/[\Q$VISUAL_SEPARATOR\E]+//g;
+		$str2 =~ s/^tel\://;
+		if( $str2 =~ /^\+($IDD_RE)/ )
+		{
+			my $idd = $1;
+			if( CORE::exists( $COUNTRIES->{ $idd } ) )
+			{
+				my $idds = $COUNTRIES->{ $idd }->[0]->{ 'idd' };
+				foreach my $thisIdd ( @$idds )
+				{
+					my $check = $thisIdd;
+					$check =~ s/\D//g;
+					if( $check eq $idd )
+					{
+						$temp->{ 'context' } = '+' . $thisIdd;
+						last;
+					}
+				}
+			}
+		}
+	}
+	
 	## The component name for each match
 	@$temp{ @names } = @matches;
 	$temp->{ 'params' } = $temp->{ 'params1' } ? $temp->{ 'params1' } : $temp->{ 'params2' } if( !length( $temp->{ 'params' } ) );
-	( $temp->{ 'context' } = $temp->{ 'context' } ) =~ s/;[^=]+=(.*?)$/$1/gs;
+	$temp->{ 'context' } =~ s/;[^=]+=(.*?)$/$1/gs;
 	
 	my $hash  = {
 	'original'		=> ( $orig ne $str ) ? $orig : $temp->{ 'all' },
@@ -179,6 +203,7 @@ sub new
 		elsif( lc( $p ) eq 'phone_context' )
 		{
 			$hash->{ 'context' } = $v;
+			$temp->{ '_has_context_param' } = 1;
 		}
 		else
 		{
@@ -188,7 +213,7 @@ sub new
 	$self->{ 'private' } = $priv;
 	$self->{ 'ext' } = $temp->{ 'ext' } if( !length( $hash->{ 'ext' } ) && !$self->{ 'is_vanity' } );
 	$self->{ 'ext' } =~ s/\D//gs;
-	$self->{ '_prepend_context' } = 0;
+	$self->{ '_prepend_context' } = $temp->{ '_has_context_param' } ? 0 : 1;
 	return( $self );
 }
 
@@ -202,14 +227,14 @@ sub as_string
 	push( @params, sprintf( "isub=%s", $self->{ 'isdn_subaddress' } ) ) if( length( $self->{ 'isdn_subaddress' } ) );
 	if( length( $self->{ 'context' } ) )
 	{
-		if( $self->{ '_prepend_context' } && $self->{ 'subscriber' } !~ /^\+\d+/ )
-		{
-			@uri = ( 'tel:' . $self->{ 'context' } . '.' . $self->{ 'subscriber' } );
-		}
-		else
+		if( !$self->{ '_prepend_context' } )
 		{
 			push( @params, sprintf( "phone-context=%s", $self->{ 'context' } ) );
 		}
+		elsif( $self->{ 'subscriber' } !~ /^\+\d+/ && $self->{ 'context' } !~ /[a-zA-Z]+/ )
+		{
+			@uri = ( 'tel:' . $self->{ 'context' } . '.' . $self->{ 'subscriber' } );
+		}
 	}
 	my $priv = $self->{ 'private' };
 	foreach my $k ( sort( keys( %$priv ) ) )
@@ -250,6 +275,8 @@ sub canonical
 	$uri->ext( $self->{ 'ext' } ) if( length( $self->{ 'ext' } ) );
 	$uri->isub( $self->{ 'isdn_subaddress' } ) if( length( $self->{ 'isdn_subaddress' } ) );
 	$uri->context( $self->{ 'context' } ) if( length( $self->{ 'context' } ) );
+	$uri->{_has_context_param} = $self->{_has_context_param};
+	$uri->{_prepend_context} = $self->{_prepend_context};
 	my $priv = $self->{ 'private' };
 	%{$uri->{ 'private' }} = %$priv;
 	return( $uri );
@@ -301,6 +328,8 @@ sub context
 			return( undef() );
 		}
 		delete( $self->{ 'cache' } );
+		$self->{ '_has_context_param' } = 1;
+		$self->{ '_prepend_context' } = 0;
 		$self->{ 'context' } = $str;
 	}
 	return( $self->{ 'context' } );
@@ -322,8 +351,7 @@ sub country
 	{
 		next if( length( $k ) > length( $idd ) );
 		## We found a match
-		## Although it is possible that more than one country match an international dialing code, we just return the first one found.
-		## Not a perfect solution though...
+		## We return all the countries that match the international prefix
 		if( substr( $idd, 0, length( $k ) ) eq $k )
 		{	
 			my $ref = $hash->{ $k };
@@ -579,6 +607,9 @@ sub _load_countries
 			}
 		}
 		$COUNTRIES = $hash;
+		my @list = sort{ $b <=> $a } keys( %$hash );
+		my $re_list = join( '|', @list );
+		$IDD_RE = qr{(?:$re_list)};
 	}
 }
 
@@ -763,7 +794,7 @@ It returns true or false.
 
 =item B<is_vanity>()
 
-Returns true or false whether the telephone number is a vanity number, such as C+1-800-LAWYERS>.
+Returns true or false whether the telephone number is a vanity number, such as C<+1-800-LAWYERS>.
 
 =item B<isub>( [ ISDN SUBADDRESS ] )
 
@@ -796,6 +827,14 @@ This is a read-only method. It returns the type of the telephone number. The typ
 
 =back
 
+=head1 SEE ALSO
+
+List of country calling codes: E<lt>F<https://en.wikipedia.org/wiki/List_of_country_calling_codes>E<gt>
+
+=head1 CREDITS
+
+Credits to Thiago Berlitz Rondon for the initial version.
+
 =head1 COPYRIGHT
 
 Copyright (c) 2016-2018 Jacques Deguest E<lt>F<jack@deguest.jp>E<gt>
@@ -819,12 +858,12 @@ IT;ITA;Italy;39
 RO;ROU;Romania;40
 CH;CHE;Switzerland;41
 AT;AUT;Austria;43
-GG;GGY;Guernsey;44
+GG;GGY;Guernsey;44-1481
 GB;GBR;United Kingdom;44
 DK;DNK;Denmark;45
 SE;SWE;Sweden;46
 NO;NOR;Norway;47
-SJ;SJM;Svalbard and Jan Mayen;47
+SJ;SJM;Svalbard and Jan Mayen;47-79
 PL;POL;Poland;48
 DE;DEU;Germany;49
 PE;PER;Peru;51
@@ -933,6 +972,7 @@ BY;BLR;Belarus;375
 AD;AND;Andorra;376
 MC;MCO;Monaco;377
 SM;SMR;San Marino;378
+VA;VAT;Vatican City;379,39-06-698
 UA;UKR;Ukraine;380
 RS;SRB;Serbia;381
 ME;MNE;Montenegro;382
@@ -964,7 +1004,7 @@ PY;PRY;Paraguay;595
 MQ;MTQ;Martinique;596
 SR;SUR;Suriname;597
 UY;URY;Uruguay;598
-BQ;BES;Bonaire, Sint Eustatius and Saba;599
+BQ;BES;Bonaire, Sint Eustatius and Saba;599-3,599-4,599-7
 TL;TLS;Timor-Leste;670
 AQ;ATA;Antarctica;672
 NF;NFK;Norfolk Island;672
@@ -1049,8 +1089,8 @@ JE;JEY;Jersey;44-1534
 IM;IMN;Isle of Man;44-1624
 FK;FLK;Falkland Islands;500
 CW;CUW;Curaçao;599-9
-CX;CXR;Christmas Island;61-8
-CC;CCK;Cocos (Keeling) Islands;61-8
+CX;CXR;Christmas Island;61-8-9164
+CC;CCK;Cocos (Keeling) Islands;61-8-9162
 PN;PCN;Pitcairn;64
 RU;RUS;Russia;7
 KZ;KAZ;Kazakhstan;7-6,7-7
